/*  File: analyser-ZQ.cc
 *  Project: Deductor
 *  Created by Zoltán Nagy on 9/8/13.
 *  Modified by D. Soper on 14 September 2016.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
/*   local headers  */
#include "analyzer-ZQ.h"

void UserZQ::initfunc()
{
  /*   Create object to calculate the total cross section   */
  Hist0D::phys(1, "Total cross section");
  
  /*   Create a histogram to calculate the Q-distribution   */
  unsigned int nbins = 67;
  double Qmin = 350.0_GeV, Qmax = 7050.0_GeV;
  auto bins = spacing<hist1d>::linear(nbins, Qmin, Qmax);
  
  Hist1D::phys(1, "$Z/\\gamma$ $Q$ distribution - Full", bins);
  Hist1D::phys(2, "$Z/\\gamma$ $Q$ distribution - NoDelta", bins);
  Hist1D::phys(3, "$Z/\\gamma$ $Q$ distribution - ThresholdMain", bins);
  Hist1D::phys(4, "$Z/\\gamma$ $Q$ distribution - Std", bins);
  Hist1D::phys(5, "$Z/\\gamma$ $Q$ distribution - LO", bins);
  Hist1DR4::phys(6, "$Z/\\gamma$ $Q$ distribution - ratios", bins);
  count = 0;
}

void UserZQ::userfunc(const shower_history<hhc>& history)
{
  bool newHard = true;
  /*   the shower stage after the shower evolution   */
  for(auto it = history.cbegin(); it != history.cend(); ++it)
  {
    auto& p = it->state;
    auto& hard = it.hard();
    
    /*   Calculate weights at the end of the shower   */
    constexpr double tonb = 389379.338;
    auto& weights = it->weights;
    // The weight for a probability conserving shower
    // including factor to use NLO pdfs at hard scale.
    double wStd = weights.unitary.main;
           wStd*= weights.unitary.pdf_ratio*color_weight(*it)*tonb;
    // The main threshold factor, omitting P^{reg} term and the Delta term.
    double wThresholdMain = wStd*weights.threshold.main;
    // Now include the P^{reg} term.
    double wNoDelta = wThresholdMain*weights.threshold.reg;
    // Now include the Delta term also, giving the full result.
    double wFull = wNoDelta*weights.threshold.delta;
    
    /*   Calculate weight at the start of the shower   */
    double wHard = 0.0;
    if (newHard)
    {
      auto hweights = hard.weights;
      wHard = hweights.unitary.main;
      wHard *= hweights.unitary.pdf_ratio*color_weight(hard)*tonb;
    }
    newHard = false;
    /* So if there are N final states for one hard interaction,
       wHard will be nonzero only once. */
    
    std::array<double,4> myweights4;
    myweights4[0] = wFull;
    myweights4[1] = wThresholdMain;
    myweights4[2] = wNoDelta;
    myweights4[3] = wStd;
    
    /*   Total cross section  */
    Hist0D::physfill(1, wFull);
    
    /*   mass of the ee pair   */
    auto pee = p[-2].momentum + p[-3].momentum;
    auto eemass = pee.mag();
    
    /*   Z/gamma-mass distribution   */
    if(eemass > 350.0 && eemass < 7050.0) {
      Hist1D::physfill(1, wFull, dirac{}, eemass);
      Hist1D::physfill(2, wNoDelta, dirac{}, eemass);
      Hist1D::physfill(3, wThresholdMain, dirac{}, eemass);
      Hist1D::physfill(4, wStd, dirac{}, eemass);
      Hist1D::physfill(5, wHard, dirac{}, eemass);
      Hist1DR4::physfill(6, myweights4, dirac{}, eemass);
    }
  }
}
