/*  File: analyser-jets.h
 *  Project: Deductor
 *  Created by Zoltán Nagy on 9/8/13.
 *  Modified by D. Soper on 1 September 2016
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __analyzer_jets_h__
#define __analyzer_jets_h__

/*   project headers  */
#include <user.h>

/*   other includes   */
#include "fastjet.h"

/*   Define some namespaces  */
using namespace std;
using namespace duct;
using namespace duct::distpoint;

/*   Narrowing the distbook template   */
using DistVoid = distbook<double, void>;
using DistHist1d = distbook<double, hist1d>;

/*   
 *   This is the declaration of the analyzer class.
 */
class UserJets : public basic_user<hhc, DistVoid, DistHist1d>
{
  /*   useful aliases  */
  using Hist0D = Phys<DistVoid>;
  using Hist1D = Phys<DistHist1d>;

public:
  /*    constructors     */
  UserJets(const char *n, fj::JetDefinition&& jd, const duration_type& d = 2.0min)
  : jetDef{std::move(jd)} { this->param(n, d);}
  
  UserJets(const char *n, const fj::JetDefinition& jd, const duration_type& d = 2.0min)
  : jetDef{jd} { this->param(n, d);}
  
  
  /*   User defined init function. It is called once at the begining of the calculation.  */
  void initfunc();
  
  /*   The analyser routine. It analyses the events and fill the histograms.   */
  void userfunc(const shower_history<hhc>&);
  
  /*   For fastjet analysis - select algorithm and parameters.    */
  fj::JetDefinition jetDef;
  
  /*   Predefined vectors of Pseudojets     */
  mutable std::vector<fj::PseudoJet> theparticles;
  mutable std::vector<fj::PseudoJet> incJets;
};

#endif


