/*  File: mod-jets.cc
 *  Project: Deductor
 *  Created by Zoltán Nagy on 9/8/13.
 *  Modified by D. Soper on 27 September 2016.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*   Deductor headers  */
#include <deductor.h>
#include <proc-hhcjets.h>


/*   Analyser headers  */
#include "analyzer-jets.h"

template<class _Ordering>
void my_jet_module(const deductor_input& info)
{
  /*  Deductor object  */
  deductor<hhc> kernel{qcd::ct14n};
  
  /*  Hard process     */
  auto hard = [=](const model_ref& mdl) -> hhcjets
  {
    int proton = 2212;
    double Ecm = 13.0_TeV;
    double scalefac = 1.0;
    auto pTlist = {250.0, 700.0, 1000.0, 1500.0, 2000.0, 2500.0, 3000.0, 3500.0,
      4000.0, 4500.0, 5050.0};
    
    return {mdl, Ecm, pTlist, scalefac, proton, proton};
  };
  
  /*   Jet definition    */
  fj::JetDefinition jetDef{fj::antikt_algorithm, 0.4};
  
  /*   Jet cross section pT distribution */
  UserJets *Jets = new UserJets{"Jets", jetDef};
  
  /* Create the shower process, start the calculation.   */
  
  shower_engine_opts opts;
  opts.max_color_suppression = 0u;    // The default is 0u.
  opts.allow_color_branching = false; // The default is true.
  opts.threshold.psi_min = 0.01;      // The default is 0.01.
  
  kernel("JetCalculations", opts, hard, {Jets});
  cout << "Calculation started." << endl;
}

/*   Always export your module functions!   */
export_my_module jetcalc_module = {
  {module_type::process, "Jets-13TeV", my_jet_module<ordering::lambda>}
};
